import requests
import json
from kudapy.algorithms.aes_algorithm import  aes_encrypt, aes_decrypt
from kudapy.algorithms.rsa_algorithm import rsa_encrypt, rsa_decrypt
from kudapy.utils import generate_id



def kuda(public_key, private_key, client_key):
    def make_kuda_request(service_type, request_ref, data=None):
        short_id = generate_id(5, is_alphanum=True)
        password = f"{client_key}-{short_id}"

        endpoint = "https://kudaopenapi.azurewebsites.net/v1"

        payload = {
                "serviceType": service_type,
                "requestRef": request_ref,
                "data": data
                }

        # aes encryption of payload with password
        payload = json.dumps(payload)
        encrypted_payload = aes_encrypt(payload, password)
        encrypted_payload_json = json.loads(encrypted_payload)
        iv = encrypted_payload_json['iv']
        ciphertext = encrypted_payload_json['ciphertext']


        # rsa encryption of password wih public key
        encrypted_password = rsa_encrypt(password, public_key)

        headers = {
            "password": encrypted_password,
        }
        payload_ = {
            "data":ciphertext
        }
        encrypted_response = requests.post(
            endpoint, json=payload_,
            headers=headers
            )

        # RSA decrypt password with our privateKey
        encrypted_response = encrypted_response.text
        encrypted_response = json.loads(encrypted_response)
        encrypted_password = encrypted_response["password"]
        encrypted_data = encrypted_response["data"]

        decrypted_password = rsa_decrypt(encrypted_password, private_key)

        #AES decrypt data with password
        decrypted_data = aes_decrypt(encrypted_data, decrypted_password)
        decrypted_data = str(decrypted_data, 'utf-8')
        print("decrypted_data", decrypted_data)

    return make_kuda_request


from Crypto.PublicKey import RSA
from Crypto import Random
from Crypto.Cipher import PKCS1_OAEP
from Crypto.Cipher import PKCS1_v1_5
import base64


def rsa_encrypt(message, public_key):
    message = str.encode(message)
    rsa_public_key = RSA.importKey(public_key)
    rsa_public_key = PKCS1_v1_5.new(rsa_public_key)
    encrypted_text = rsa_public_key.encrypt(message)
    encrypted_text = base64.b64encode(encrypted_text)
    return encrypted_text

def rsa_decrypt(encrypted_text, private_key):
    encrypted_text = base64.b64decode(encrypted_text)
    sentinel = "an rsa_decryption error occured"
    rsa_private_key = RSA.importKey(private_key)
    rsa_private_key = PKCS1_v1_5.new(rsa_private_key)
    decrypted_text = rsa_private_key.decrypt(encrypted_text, sentinel)
    return decrypted_text



import hashlib
import json
from base64 import b64encode, b64decode
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad


def aes_encrypt(data, password):
    data = bytes(data, 'UTF-8')
    password = bytes(password, 'UTF-8')
    derived_key = hashlib.pbkdf2_hmac('sha1', password, b'randomsalt', 1000, dklen=32)
    iv = hashlib.pbkdf2_hmac('sha1', password, b'randomsalt', 1000, dklen=16)
    cipher = AES.new(derived_key, AES.MODE_CBC, iv)
    ct_bytes = cipher.encrypt(pad(data, AES.block_size))
    iv = b64encode(cipher.iv).decode('utf-8')
    ct = b64encode(ct_bytes).decode('utf-8')
    result = json.dumps({'iv':iv, 'ciphertext':ct})
    return result



# encrypted_payload = aes_encrypt(data, password)
# print(encrypted_payload)


def aes_decrypt(encrypted_data, password):
    #try:
    derived_key = hashlib.pbkdf2_hmac('sha1', password, b'randomsalt', 1000, dklen=32)
    iv = hashlib.pbkdf2_hmac('sha1', password, b'randomsalt', 1000, dklen=16)
    ct = b64decode(encrypted_data)
    cipher = AES.new(derived_key, AES.MODE_CBC, iv)
    pt = unpad(cipher.decrypt(ct), AES.block_size)
    return pt
    #except (ValueError, KeyError):
    #   print("Incorrect decryption")


import random
import string


def generate_id(n, is_alphanum=True):
    """
    generate a random id of length n
    is_alphanum gives a random string of alphanumerals by default
    set is_alphanum to False for a random string of numbers

    """
    _id = ''.join(["{}".format(random.randint(0, 9)) for num in range(0, n)])
    if is_alphanum:
        letters_and_digits = string.ascii_letters + string.digits
        _id = ''.join(random.choice(letters_and_digits) for i in range(0, n))
    return _id

def load_private_key():
    """
    load RSA private key in pem format
    """
    with open("./private.pem", "rb") as key_file:
        private_key = key_file.read()
    return private_key

def load_public_key():
    """
    load RSA public key in pem format
    """
    with open("./public.pem", "rb") as key_file:
        public_key = key_file.read()
    return public_key



from kudapy import version
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="kudapy",
    version="1.0.3",
    author=version.__author__,
    author_email="daleentontech@gmail.com",
    description="Python wrapper for making secure requests to Kuda API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/daleentontech/kudapy",
    packages=setuptools.find_packages(),
    install_requires=['requests', 'pycryptodome'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    project_urls={
        'Source': 'https://github.com/daleentontech/kudapy/',
    },
)